# -*- coding: utf-8 -*-
import scrapy, re, json
from datetime import date
from prensaGrafica.items import NoticiasItem

"""
MEDIO:
La Prensa Gráfica, El Salvador
USO:
// Si se quiere obtener todas las noticias desde las más actuales hasta las más antiguas. //
scrapy crawl noticias --nolog -s filename=noticias.json
-------------------------------------------------------------------------------------------------
// Si se quiere obtener todas las noticias desde las más actuales hasta una fecha específica. //
scrapy crawl noticias --nolog -s filename=noticias.json -a year=2018 -a month=2 -a day=29
-------------------------------------------------------------------------------------------------
Después será necesario hacer uso del archivo parse_date_files.py para que las noticias contenidas
en noticias.json sean separadas en archivos por fecha.
"""

TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)

DAT_RE = re.compile(r'-\d{8}-')


class ImportantData(scrapy.Item):
    section = scrapy.Field()
    page = scrapy.Field()


class QuotesSpider(scrapy.Spider):
    name = "noticias"

    def start_requests(self):
        self.year = getattr(self, "year", None)
        self.month = getattr(self, "month", None)
        self.day = getattr(self, "day", None)

        if self.year is not None and self.month is not None and self.day is not None:
            self.stopDate = date(int(self.year), int(self.month), int(self.day))
        else:
            self.stopDate = None

        self.baseURL = "https://www.laprensagrafica.com"
        self.baseURI = self.baseURL + "/ajax/get_section_news.html?viewmore=%2Fajax%2Fget_section_news.html&page="
        self.sectionURI = "&size=6&section="

        sectionList = ["elsalvador", "departamento15", "internacionales", "deportes", "economia",
                       "opinion", "techlife", "farandula", "salud", "tendencias", "mujer", "turismo"]
        # sectionList = ["elsalvador"]

        for s in sectionList:
            yield scrapy.Request(url=self.baseURL + "/seccion/" + s, callback=self.parse)


    def parse(self, response):
        searchData = ImportantData()
        CONTINUE_SEARCHING = True
        section = response.url[response.url.rfind("/") + 1:]
        if section == "internacionales": section = "internacional"

        if self.stopDate is None:
            for link in response.xpath('//div[@class="col-main"]/article/div/h1/a/@href').extract():
                yield scrapy.Request(url=self.baseURL + link, callback=self.parse_item)

        else:
            for link in response.xpath('//div[@class="col-main"]/article/div/h1/a/@href').extract():
                res = DAT_RE.search(link)
                if res:
                    dat = res.group(0).replace("-", '')
                    newsDate = date(int(dat[:4]), int(dat[4:6]), int(dat[6:]))
                    if newsDate >= self.stopDate:
                        yield scrapy.Request(url=self.baseURL + link, callback=self.parse_item)

                    else:
                        CONTINUE_SEARCHING = False
                        break

        if CONTINUE_SEARCHING:
            page = 2
            url = self.baseURI + str(page) + self.sectionURI + section
            searchData['section'] = section
            searchData['page'] = page
            request = scrapy.Request(url=url, callback=self.continue_searching)
            request.meta['item'] = searchData
            yield request


    def continue_searching(self, response):
        searchData = response.meta['item']
        CONTINUE_SEARCHING = True

        linkList = response.xpath('//article/div/h1/a/@href').extract()

        if len(linkList) > 0:
            if self.stopDate is None:
                for link in linkList:
                    yield scrapy.Request(url=self.baseURL + link, callback=self.parse_item)

            else:
                for link in linkList:
                    res = DAT_RE.search(link)
                    if res:
                        dat = res.group(0).replace("-", '')
                        newsDate = date(int(dat[:4]), int(dat[4:6]), int(dat[6:]))
                        if newsDate >= self.stopDate:
                            yield scrapy.Request(url=self.baseURL + link, callback=self.parse_item)

                        else:
                            CONTINUE_SEARCHING = False
                            break

        else:
            CONTINUE_SEARCHING = False

        if CONTINUE_SEARCHING:
            searchData['page'] += 1
            page = searchData['page']
            section = searchData['section']
            url = self.baseURI + str(page) + self.sectionURI + section
            request = scrapy.Request(url=url, callback=self.continue_searching)
            request.meta['item'] = searchData
            yield request


    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        "La fecha obtenida ya incluye formato y zona horaria"
        newsData = remove_tags(response.xpath('//script[@type="application/ld+json"]').extract_first())
        newsDict = json.loads(newsData)
        item['date'] = newsDict['datePublished']
        item['title'] = newsDict['headline']

        try:
            topic = response.xpath('//div[@class="news-line"]/a/text()').extract()[-1]
        except:
            topic = None
        item['topic'] = topic

        author = response.css('div.content-author').xpath('./p/meta[@itemprop="name"]/@content').extract_first()
        if author is not None:
            item['author'] = author

        for p in response.css('div.news-body').css('p').extract():
            text += remove_tags(p) + "\n"

        item['text'] = text.strip()

        item['url'] = response.url

        yield item
